@@ -2231,8 +2231,13 @@ var ObservableJupyterIframe = (function (exports) {
     throw variable_undefined;
   }
 
+  function variable_stale() {
+    throw variable_stale;
+  }
+
   function variable_rejector(variable) {
     return function(error) {
+      if (error === variable_stale) throw error;
       if (error === variable_undefined) throw new RuntimeError(variable._name + " is not defined", variable._name);
       if (error instanceof Error && error.message) throw new RuntimeError(error.message, variable._name);
       throw new RuntimeError(variable._name + " could not be resolved", variable._name);
@@ -2343,6 +2348,11 @@ var ObservableJupyterIframe = (function (exports) {
       this._name = name;
     }
 
+    // If this redefined variable was previously evaluated, invalidate it. (If the
+    // variable was never evaluated, then the invalidated value could never have
+    // been exposed and we can avoid this extra work.)
+    if (this._version > 0) ++this._version;
+
     runtime._updates.add(this);
     runtime._compute();
     return this;
@@ -2422,8 +2432,18 @@ var ObservableJupyterIframe = (function (exports) {
       v._observer = true;
       this._runtime._dirty.add(v);
     }
-    await this._runtime._compute();
-    return v._promise;
+    return module_revalue(this._runtime, v);
+  }
+
+  // If the variable is redefined before its value resolves, try again.
+  async function module_revalue(runtime, variable) {
+    await runtime._compute();
+    try {
+      return await variable._promise;
+    } catch (error) {
+      if (error === variable_stale) return module_revalue(runtime, variable);
+      throw error;
+    }
   }
 
   function module_derive(injects, injectModule) {
@@ -2506,7 +2526,9 @@ var ObservableJupyterIframe = (function (exports) {
     return variable._name;
   }
 
-  const frame = typeof requestAnimationFrame === "function" ? requestAnimationFrame : setImmediate;
+  const frame = typeof requestAnimationFrame === "function" ? requestAnimationFrame
+    : typeof setImmediate === "function" ? setImmediate
+    : f => setTimeout(f, 0);
 
   var variable_invalidation = {};
   var variable_visibility = {};
@@ -2741,7 +2763,7 @@ var ObservableJupyterIframe = (function (exports) {
 
     // Compute the initial value of the variable.
     function define(inputs) {
-      if (variable._version !== version) return;
+      if (variable._version !== version) throw variable_stale;
 
       // Replace any reference to invalidation with the promise, lazily.
       for (var i = 0, n = inputs.length; i < n; ++i) {
@@ -2766,8 +2788,8 @@ var ObservableJupyterIframe = (function (exports) {
     // already have been invalidated here, in which case we need to terminate the
     // generator immediately!
     function generate(value) {
+      if (variable._version !== version) throw variable_stale;
       if (generatorish(value)) {
-        if (variable._version !== version) return void value.return();
         (invalidation || variable_invalidator(variable)).then(variable_return(value));
         return variable_generate(variable, version, value);
       }
@@ -2775,11 +2797,10 @@ var ObservableJupyterIframe = (function (exports) {
     }
 
     promise.then((value) => {
-      if (variable._version !== version) return;
       variable._value = value;
       variable._fulfilled(value);
     }, (error) => {
-      if (variable._version !== version) return;
+      if (error === variable_stale) return;
       variable._value = undefined;
       variable._rejected(error);
     });
@@ -2787,12 +2808,13 @@ var ObservableJupyterIframe = (function (exports) {
 
   function variable_generate(variable, version, generator) {
     const runtime = variable._module._runtime;
+    let currentValue; // so that yield resolves to the yielded value
 
     // Retrieve the next value from the generator; if successful, invoke the
     // specified callback. The returned promise resolves to the yielded value, or
     // to undefined if the generator is done.
     function compute(onfulfilled) {
-      return new Promise(resolve => resolve(generator.next())).then(({done, value}) => {
+      return new Promise(resolve => resolve(generator.next(currentValue))).then(({done, value}) => {
         return done ? undefined : Promise.resolve(value).then(onfulfilled);
       });
     }
@@ -2803,13 +2825,15 @@ var ObservableJupyterIframe = (function (exports) {
     // successful, reject the variable, compute downstream variables, and return.
     function recompute() {
       const promise = compute((value) => {
-        if (variable._version !== version) return;
+        if (variable._version !== version) throw variable_stale;
+        currentValue = value;
         postcompute(value, promise).then(() => runtime._precompute(recompute));
         variable._fulfilled(value);
         return value;
       });
       promise.catch((error) => {
-        if (variable._version !== version) return;
+        if (error === variable_stale) throw error;
+        if (variable._version !== version) throw variable_stale;
         postcompute(undefined, promise);
         variable._rejected(error);
       });
@@ -2827,7 +2851,8 @@ var ObservableJupyterIframe = (function (exports) {
     // When retrieving the first value from the generator, the promise graph is
     // already established, so we only need to queue the next pull.
     return compute((value) => {
-      if (variable._version !== version) return;
+      if (variable._version !== version) throw variable_stale;
+      currentValue = value;
       runtime._precompute(recompute);
       return value;
     });
